#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
from __future__ import annotations
from abc import  abstractmethod, ABC
from enum import Enum
from io import BytesIO
from typing import List, Dict
from pandas import DataFrame

from soup_files import File
from convert_stream.models.m_pdf_page import LibraryPDF, ABCPagePdf


class ABCDocumentPdf(ABC):
    def __init__(self, library:LibraryPDF=LibraryPDF.PYPDF):
        self.__pages:List[ABCPagePdf] = []
        self.__num_pages:int = 0
        self.library:LibraryPDF = library
        self.__max_num_pages:int= 4000

    @property
    def max_num_pages(self) -> int:
        return self.__max_num_pages

    @max_num_pages.setter
    def max_num_pages(self, new:int):
        if isinstance(new, int):
            self.__max_num_pages = new
         
    @property
    def num_pages(self) -> int:
        return self.__num_pages
    
    @num_pages.setter
    def num_pages(self, new:int):
        if not isinstance(new, int):
            print(f'Número inválido...')
            return
        self.__num_pages = new

    @property
    def pages(self) -> List[ABCPagePdf]:
        return  self.__pages

    @pages.setter
    def pages(self, new:List[ABCPagePdf]):
        if not isinstance(new, list):
            raise ValueError(f'{__class__.__name__}Erro: Use List[PagePdf], não {type(new)}')
        self.__pages = new
        
    @abstractmethod
    def is_null(self) -> bool:
        pass

    @abstractmethod
    def add_file_pdf(self, file:File):
        pass

    @abstractmethod
    def add_page(self, page:ABCPagePdf) -> bool:
        pass

    @abstractmethod
    def add_pages(self, pages:List[ABCPagePdf]):
        pass

    @abstractmethod
    def add_bytes_page(self, page_bytes:bytes):
        pass
    
    @abstractmethod
    def add_bytes_file_pdf(self, file_bytes:BytesIO):
        pass
    
    @abstractmethod
    def add_file_url(self, url:str):
        pass
        
    @abstractmethod
    def remove_page(self, page_number:int) -> None:
        pass
    
    @abstractmethod
    def remove_pages(self, num_pages:List[int]):
        pass
        
    @abstractmethod
    def rotate_page(self, page_number: int, degrees: int):
        pass
    
    @abstractmethod
    def set_paisagem(self):
        pass
        
    @abstractmethod
    def to_file_pdf(self, file:File):
        pass
    
    @abstractmethod
    def to_data(self, separator='\n') -> DataFrame:
        pass
    
    @abstractmethod
    def to_list(self, separator='\n') -> List[str]:
        pass
    
    @abstractmethod
    def to_excel(self, file:File, *, separator:str='\n'):
        pass
    
    @abstractmethod
    def clear(self):
        pass

    @abstractmethod
    def get_page_number(self, num:int) -> ABCPagePdf:
        pass
    @abstractmethod
    def get_pages_numbers(self, numbers:List[int]) -> List[ABCPagePdf]:
        pass

    @abstractmethod
    def get_document_with_text(self, text:str) -> ABCDocumentPdf:
        pass

    
        
 
    
    








