#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
from __future__ import annotations
from abc import  abstractmethod, ABC
from enum import Enum
from typing import List, Dict
from pandas import DataFrame

from soup_files import File
from convert_stream.models.m_text import TableText


class LibraryPDF(Enum):
    """Enumerar as libs para manipulação de PDF"""
    PYPDF = 'pypdf2'
    FITZ = 'fitz'
    CANVAS = 'canvas'
    PILPDF = 'pil'


# Representação de uma página PDF.
class ABCPagePdf(ABC):
    """Abstração de uma página PDF"""

    def __init__(self):
        super().__init__()
        self.page:object = None
        self.page_number: int = 0
        self.current_library: LibraryPDF = None

    @abstractmethod
    def to_string(self) -> str:
        pass
    
    @abstractmethod
    def to_list(self, separator:str='\n') -> List[str]:
        pass
    
    @abstractmethod
    def to_table(self, separator:str='\n') -> TableText:
        pass

    @abstractmethod
    def to_bytes(self) -> bytes:
        pass
    
    @abstractmethod
    def is_paisagem(self) -> bool:
        pass

    @abstractmethod
    def set_paisagem(self):
        pass
    
    @abstractmethod
    def set_page_pypdf2(self):
        pass
    
    @abstractmethod
    def set_page_fitz(self):
        pass

    @abstractmethod
    def rotate(self, degrees: int):
        pass 
    
    @classmethod
    def create_from_page_bytes(cls, page_bytes:bytes) -> ABCPagePdf:
        pass
    
    @classmethod
    def create_from_page_fitz(cls, page:object) -> ABCPagePdf:
        pass
    
    @classmethod
    def create_from_page_pypdf2(cls, page:object) -> ABCPagePdf:
        pass

    








